let logs = []

const getDefaultRetries = () => {
  return Cypress.env('RETRIES')
}

const _clone = Cypress.mocha._mocha.Mocha.Test.prototype.clone
Cypress.mocha._mocha.Mocha.Test.prototype.clone = function() {
  if (this.trueFn) {
    this.fn = this.trueFn
  }
  const ret = _clone.apply(this, arguments)
  ret.id = this.id
  ret.err = null
  debug('clone test')
  logs.forEach(log => {
    log.set({
      state: log.get().state + ' ignored' + ' retry-' + ret._currentRetry
    })
  })
  logs = []

  return ret
}

Cypress.on('log:added', (attr, log) => {
  logs.push(log)
})

const _onRunnableRun = Cypress.runner.onRunnableRun
Cypress.runner.onRunnableRun = function(runnableRun, runnable, args) {
  debug('_onRunnableRun')

  const r = runnable
  const isHook = r.type === 'hook'
  const isAfterHook = isHook && r.hookName.match(/after/)
  const isAfterAllHook = isHook && r.hookName.match(/after all/)
  const isBeforeHook = isHook && r.hookName.match(/before/)
  const test = r.ctx.currentTest || r

  if (test._currentRetry === 0 && logs.testId !== test.id) {
    logs = []
    logs.testId = test.id
  }

  const next = args[0]

  if (isAfterAllHook) {
    test.err = null
    if (test.state !== 'failed') {
      test.state = 'passed'
    }
  }

  debug('on:', r.title)

  if (
    isHook &&
    r.ctx.currentTest &&
    r.ctx.currentTest.trueFn &&
    !isAfterAllHook
  ) {
    debug('already failed, skipping this hook')
    return next.call(this)
  }
  debug('running')

  const onNext = function(err) {
    debug(runnable.title, 'onNext')

    const fail = function() {
      return next.call(this, err)
    }
    const noFail = function() {
      test.err = null
      return next.call(this)
    }

    if (err) {
      if (test._retries === -1) {
        test._retries = getDefaultRetries()
      }

      if (isBeforeHook && test._currentRetry < test._retries) {
        test.trueFn = test.fn
        test.fn = function() {
          throw err
        }
        return noFail()
      }
    }
    return fail()
  }
  args[0] = onNext

  return _onRunnableRun.apply(this, [runnableRun, runnable, args])
}

addGlobalStyle(/*css*/ `
.command-state-retry {
  color: orange
}
.command-state-retry .command-number {
  /* display:none */
}
.command-state-retry .command-method span{
  background-color: #FCB827;
  color: white;
  border-radius: 2px;
  padding: 0 3px;
  font-size: 11px;
  display: inline-block;
  height: 14px;
  line-height: 16px;
}

.command.ignored {
  opacity: 0.3
}
.command.ignored:hover  {
  opacity: 1
}
.command.ignored span .command-wrapper {
  border-left: 4px solid orange
}
.command.ignored .command-number span{
  display: none;
}

.command.ignored .command-number:before {
  font: normal normal normal 12px/1 FontAwesome;
  content: " ";
  color: orange;
}
/* .command.ignored.retry-0 .command-number:before {
  content: "";
}
.command.ignored.retry-1 .command-number:before {
  content: "2";
}
.command.ignored.retry-2 .command-number:before {
  content: "3";
} */

/* .command.ignored.retry-1 {
  opacity: 0.2
}
.command.ignored.retry-2 {
  opacity: 0.3
} */

.runnable-passed .test-error {
  display:none
}


`)

function addGlobalStyle(css) {
  var head, style
  head = window.top.document.getElementsByTagName('head')[0]
  if (!head) {
    return
  }
  style = window.top.document.createElement('style')
  style.type = 'text/css'
  style.innerHTML = css
  head.appendChild(style)
}

const debug = function() {
  // console.log.apply(this, arguments)
}